@@ -1,16 +1,27 @@
+// Copyright 2018 the Kurbo Authors
+// SPDX-License-Identifier: Apache-2.0 OR MIT
+
 //! SVG path representation.
 
+use alloc::vec::Vec;
+use core::f64::consts::PI;
+use core::fmt::{self, Display, Formatter};
+// MSRV: Once our MSRV is 1.81, we can switch to `core::error`
+#[cfg(feature = "std")]
 use std::error::Error;
-use std::f64::consts::PI;
-use std::fmt::{self, Display, Formatter};
+#[cfg(feature = "std")]
 use std::io::{self, Write};
 
 use crate::{Arc, BezPath, ParamCurve, PathEl, PathSeg, Point, Vec2};
 
+#[cfg(not(feature = "std"))]
+use crate::common::FloatFuncs;
+
 // Note: the SVG arc logic is heavily adapted from https://github.com/nical/lyon
 
 /// A single SVG arc segment.
 #[derive(Clone, Copy, Debug)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct SvgArc {
     /// The arc's start point.
@@ -29,7 +40,7 @@ pub struct SvgArc {
 }
 
 impl BezPath {
-    /// Create a BezPath with segments corresponding to the sequence of
+    /// Create a `BezPath` with segments corresponding to the sequence of
     /// `PathSeg`s
     pub fn from_path_segments(segments: impl Iterator<Item = PathSeg>) -> BezPath {
         let mut path_elements = Vec::new();
@@ -56,6 +67,7 @@ impl BezPath {
     ///
     /// The current implementation doesn't take any special care to produce a
     /// short string (reducing precision, using relative movement).
+    #[cfg(feature = "std")]
     pub fn to_svg(&self) -> String {
         let mut buffer = Vec::new();
         self.write_to(&mut buffer).unwrap();
@@ -63,15 +75,19 @@ impl BezPath {
     }
 
     /// Write the SVG representation of this path to the provided buffer.
+    #[cfg(feature = "std")]
     pub fn write_to<W: Write>(&self, mut writer: W) -> io::Result<()> {
-        for el in self.elements() {
+        for (i, el) in self.elements().iter().enumerate() {
+            if i > 0 {
+                write!(writer, " ")?;
+            }
             match *el {
-                PathEl::MoveTo(p) => write!(writer, "M{} {}", p.x, p.y)?,
-                PathEl::LineTo(p) => write!(writer, "L{} {}", p.x, p.y)?,
-                PathEl::QuadTo(p1, p2) => write!(writer, "Q{} {} {} {}", p1.x, p1.y, p2.x, p2.y)?,
+                PathEl::MoveTo(p) => write!(writer, "M{},{}", p.x, p.y)?,
+                PathEl::LineTo(p) => write!(writer, "L{},{}", p.x, p.y)?,
+                PathEl::QuadTo(p1, p2) => write!(writer, "Q{},{} {},{}", p1.x, p1.y, p2.x, p2.y)?,
                 PathEl::CurveTo(p1, p2, p3) => write!(
                     writer,
-                    "C{} {} {} {} {} {}",
+                    "C{},{} {},{} {},{}",
                     p1.x, p1.y, p2.x, p2.y, p3.x, p3.y
                 )?,
                 PathEl::ClosePath => write!(writer, "Z")?,
@@ -95,6 +111,10 @@ impl BezPath {
         let mut implicit_moveto = None;
         while let Some(c) = lexer.get_cmd(last_cmd) {
             if c != b'm' && c != b'M' {
+                if path.elements().is_empty() {
+                    return Err(SvgParseError::UninitializedPath);
+                }
+
                 if let Some(pt) = implicit_moveto.take() {
                     path.move_to(pt);
                 }
@@ -228,6 +248,7 @@ impl BezPath {
 
 /// An error which can be returned when parsing an SVG.
 #[derive(Debug)]
+#[non_exhaustive]
 pub enum SvgParseError {
     /// A number was expected.
     Wrong,
@@ -235,6 +256,8 @@ pub enum SvgParseError {
     UnexpectedEof,
     /// Encountered an unknown command letter.
     UnknownCommand(char),
+    /// Encountered a command that precedes expected 'moveto' command.
+    UninitializedPath,
 }
 
 impl Display for SvgParseError {
@@ -242,11 +265,15 @@ impl Display for SvgParseError {
         match self {
             SvgParseError::Wrong => write!(f, "Unable to parse a number"),
             SvgParseError::UnexpectedEof => write!(f, "Unexpected EOF"),
-            SvgParseError::UnknownCommand(letter) => write!(f, "Unknown command, \"{}\"", letter),
+            SvgParseError::UnknownCommand(letter) => write!(f, "Unknown command, \"{letter}\""),
+            SvgParseError::UninitializedPath => {
+                write!(f, "Uninitialized path (missing moveto command)")
+            }
         }
     }
 }
 
+#[cfg(feature = "std")]
 impl Error for SvgParseError {}
 
 struct SvgLexer<'a> {
@@ -276,9 +303,9 @@ impl<'a> SvgLexer<'a> {
     fn get_cmd(&mut self, last_cmd: u8) -> Option<u8> {
         self.skip_ws();
         if let Some(c) = self.get_byte() {
-            if (c >= b'a' && c <= b'z') || (c >= b'A' && c <= b'Z') {
+            if c.is_ascii_lowercase() || c.is_ascii_uppercase() {
                 return Some(c);
-            } else if last_cmd != 0 && (c == b'-' || c == b'.' || (c >= b'0' && c <= b'9')) {
+            } else if last_cmd != 0 && (c == b'-' || c == b'.' || c.is_ascii_digit()) {
                 // Plausible number start
                 self.unget();
                 return Some(last_cmd);
@@ -310,7 +337,7 @@ impl<'a> SvgLexer<'a> {
         let mut digit_count = 0;
         let mut seen_period = false;
         while let Some(c) = self.get_byte() {
-            if c >= b'0' && c <= b'9' {
+            if c.is_ascii_digit() {
                 digit_count += 1;
             } else if c == b'.' && !seen_period {
                 seen_period = true;
@@ -323,13 +350,13 @@ impl<'a> SvgLexer<'a> {
             if c == b'e' || c == b'E' {
                 let mut c = self.get_byte().ok_or(SvgParseError::Wrong)?;
                 if c == b'-' || c == b'+' {
-                    c = self.get_byte().ok_or(SvgParseError::Wrong)?
+                    c = self.get_byte().ok_or(SvgParseError::Wrong)?;
                 }
-                if !(c >= b'0' && c <= b'9') {
+                if !c.is_ascii_digit() {
                     return Err(SvgParseError::Wrong);
                 }
                 while let Some(c) = self.get_byte() {
-                    if !(c >= b'0' && c <= b'9') {
+                    if !c.is_ascii_digit() {
                         self.unget();
                         break;
                     }
@@ -366,7 +393,7 @@ impl<'a> SvgLexer<'a> {
 
     fn get_maybe_relative(&mut self, cmd: u8) -> Result<Point, SvgParseError> {
         let pt = self.get_number_pair()?;
-        if cmd >= b'a' && cmd <= b'z' {
+        if cmd.is_ascii_lowercase() {
             Ok(self.last_pt + pt.to_vec2())
         } else {
             Ok(pt)
@@ -386,7 +413,7 @@ impl<'a> SvgLexer<'a> {
 impl SvgArc {
     /// Checks that arc is actually a straight line.
     ///
-    /// In this case, it can be replaced with a LineTo.
+    /// In this case, it can be replaced with a `LineTo`.
     pub fn is_straight_line(&self) -> bool {
         self.radii.x.abs() <= 1e-5 || self.radii.y.abs() <= 1e-5 || self.from == self.to
     }
@@ -406,8 +433,7 @@ impl Arc {
         let mut ry = arc.radii.y.abs();
 
         let xr = arc.x_rotation % (2.0 * PI);
-        let cos_phi = xr.cos();
-        let sin_phi = xr.sin();
+        let (sin_phi, cos_phi) = xr.sin_cos();
         let hd_x = (arc.from.x - arc.to.x) * 0.5;
         let hd_y = (arc.from.y - arc.to.y) * 0.5;
         let hs_x = (arc.from.x + arc.to.x) * 0.5;
@@ -480,7 +506,7 @@ impl Arc {
 
 #[cfg(test)]
 mod tests {
-    use crate::{BezPath, CubicBez, Line, ParamCurve, PathSeg, Point, QuadBez, Shape};
+    use crate::{BezPath, CubicBez, Line, ParamCurve, PathEl, PathSeg, Point, QuadBez, Shape};
 
     #[test]
     fn test_parse_svg() {
@@ -511,6 +537,28 @@ mod tests {
         assert_eq!(path.perimeter(1e-6).round(), 168.0);
     }
 
+    #[test]
+    fn test_parse_svg_uninitialized() {
+        let path = BezPath::from_svg("L10 10 100 0 0 100");
+        assert!(path.is_err());
+    }
+
+    #[test]
+    #[allow(clippy::float_cmp)]
+    fn test_parse_scientific_notation() {
+        let path = BezPath::from_svg("M 0 0 L 1e-123 -4E+5").unwrap();
+        assert_eq!(
+            path.elements(),
+            &[
+                PathEl::MoveTo(Point { x: 0.0, y: 0.0 }),
+                PathEl::LineTo(Point {
+                    x: 1e-123,
+                    y: -4E+5
+                })
+            ]
+        );
+    }
+
     #[test]
     fn test_write_svg_single() {
         let segments = [CubicBez::new(
@@ -522,7 +570,7 @@ mod tests {
         .into()];
         let path = BezPath::from_path_segments(segments.iter().cloned());
 
-        assert_eq!(path.to_svg(), "M10 10C20 20 30 30 40 40");
+        assert_eq!(path.to_svg(), "M10,10 C20,20 30,30 40,40");
     }
 
     #[test]
@@ -545,7 +593,10 @@ mod tests {
         ];
         let path = BezPath::from_path_segments(segments.iter().cloned());
 
-        assert_eq!(path.to_svg(), "M10 10C20 20 30 30 40 40C30 30 20 20 10 10");
+        assert_eq!(
+            path.to_svg(),
+            "M10,10 C20,20 30,30 40,40 C30,30 20,20 10,10"
+        );
     }
 
     #[test]
@@ -570,7 +621,7 @@ mod tests {
 
         assert_eq!(
             path.to_svg(),
-            "M10 10C20 20 30 30 40 40M50 50C30 30 20 20 10 10"
+            "M10,10 C20,20 30,30 40,40 M50,50 C30,30 20,20 10,10"
         );
     }
 
@@ -582,10 +633,10 @@ mod tests {
         let mut elements = vec![];
         let mut position = None;
 
-        let length = rng.gen_range(0, MAX_LENGTH);
+        let length = rng.gen_range(0..MAX_LENGTH);
         for _ in 0..length {
-            let should_follow: bool = rand::random();
-            let kind = rng.gen_range(0, 3);
+            let should_follow: bool = random();
+            let kind = rng.gen_range(0..3);
 
             let first = position
                 .filter(|_| should_follow)
@@ -622,7 +673,7 @@ mod tests {
     #[test]
     fn test_serialize_deserialize() {
         const N_TESTS: u32 = 100;
-        let mut rng = rand::thread_rng();
+        let mut rng = thread_rng();
 
         for _ in 0..N_TESTS {
             let vec = gen_random_path_sequence(&mut rng);
